(****************************************************************
 *  THistoryBox V1.0 by Alex Dimitrov                           *
 *  ---------------------------------                           *
 *                                                              *
 *  Usage:     Drop it on the form and set:                     *
 *             "ID" property                                    *
 *             each HistoryBox class have different ID          *
 *             Call "PutTextInHistory" to save Text any time if *
 *             you wish to save it.                             *
 *             In project source before creating forms put line *
 *               HistoryList.LoadFromFile('My History File');   *
 *             after Application.Run put line                   *
 *               HistoryList.SaveToFile('My History File');     *
 *                                                              *
 *  Comments:  Mail comments to: aldi@uni-plovdiv.bg            *
 *                                                              *
 *  Copyright: This component is freeware,                      *
 *             but keep this comment header                     *
 ****************************************************************)

unit History;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

const
  //Max number of strings to safe in HistoryList
  MaxHistoryText =1024;

type
  THistoryList = class(TStringList)
  protected
    function GetTextStr: string; override;
    procedure SetTextStr(const Value: string); override;
  end;

  THistoryBox = class(TCustomComboBox)
  private
    FID :integer;
    procedure WMCreate(var Message: TWMCreate); message WM_CREATE;
  public
    procedure PutTextInHistory;
  published
    property Style; {Must be published before Items}
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property DropDownCount;
    property Enabled;
    property Font;
    property ItemHeight;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
    property OnStartDrag;

    property ID :integer read FID write FID;
  end;

var
  HistoryList :THistoryList =nil;

procedure Register;

implementation

{ THistoryList }

function THistoryList.GetTextStr: string;
var
  I, L, Size, Count: Integer;
  P: PChar;
  S: string;
begin
  Count := GetCount;
  Size := 0;
  for I := 0 to Count - 1 do
    Inc(Size, Length(Get(I)) + Length(IntToStr(integer(GetObject(I)))) + 3);
  SetString(Result, nil, Size);
  P := Pointer(Result);
  for I := 0 to Count - 1 do
  begin
    S := Get(I)+#9+IntToStr(integer(GetObject(I)));
    L := Length(S);
    if L <> 0 then
    begin
      System.Move(Pointer(S)^, P^, L);
      Inc(P, L);
    end;
    P^ := #13;
    Inc(P);
    P^ := #10;
    Inc(P);
  end;
end;

procedure THistoryList.SetTextStr(const Value: string);
var
  P, Start: PChar;
  S: string;
  I: integer;
begin
  BeginUpdate;
  try
    Clear;
    P := Pointer(Value);
    if P <> nil then
      while P^ <> #0 do
      begin
        Start := P;
        while not (P^ in [#0, #10, #13]) do Inc(P);
        SetString(S, Start, P - Start);
        I:=pos(#9,S);
        if I=0 then
          AddObject(S,nil)
        else
          AddObject(copy(S,1,I-1),pointer(StrToInt(copy(S,I+1,10))));
        if P^ = #13 then Inc(P);
        if P^ = #10 then Inc(P);
      end;
  finally
    EndUpdate;
  end;
end;

{ THistoryBox }

procedure THistoryBox.WMCreate(var Message: TWMCreate);
var
  i :integer;
begin
  inherited;
  if HistoryList=nil then exit;
  for i:=0 to HistoryList.Count-1 do begin
    if longint(HistoryList.Objects[i])=FID then
      Items.Add(HistoryList.Strings[i]);
  end;
end;

procedure THistoryBox.PutTextInHistory;
var
  i :integer;
  s :string;
begin
  s:=Text;
  if s='' then exit;
  i:=Items.IndexOf(s);
  if i=-1 then begin
    Items.Insert(0,s);
    HistoryList.Insert(0,s);
    HistoryList.Objects[0]:=pointer(FID);
    while HistoryList.Count>MaxHistoryText do
      HistoryList.Delete(HistoryList.Count-1);
  end
  else begin
    Items.Move(i,0);
    HistoryList.Move(HistoryList.IndexOf(s),0);
    Text:=s;
  end;
end;

procedure Register;
begin
  RegisterComponents('Standard', [THistoryBox]);
end;

initialization
  HistoryList:=THistoryList.Create;

finalization
  HistoryList.Free;

end.
